package eu.lucazanini.restoreobject;

import eu.lucazanini.restoreobject.customobject.CustomObject;
import eu.lucazanini.restoreobject.customobject.CustomParcelableObject;
import eu.lucazanini.restoreobject.customobject.CustomParcelableObjectWrapper;
import eu.lucazanini.restoreobject.customobject.CustomSerializableObject;
import android.app.Activity;
import android.os.Bundle;
import android.util.Log;

public class MainActivity extends Activity {

    public final static String PARCELABLE_2D_ARRAY_KEY = "parcelable_2d_array_key";
    public final static String PARCELABLE_ARRAY_KEY = "parcelable_array_key";
    public final static String PARCELABLE_KEY = "parcelable_key";
    public final static String PARCELABLE_NAME = "parcelable object";
    public final static String RESTORED_LOG = "restored ";
    public final static String SAVED_LOG = "saved ";
    public final static String SERIALIZABLE_2D_ARRAY_KEY = "serializable_2d_array_key";

    public final static String SERIALIZABLE_ARRAY_KEY = "serializable_array_key";
    public final static String SERIALIZABLE_KEY = "serializable_key";

    public final static String SERIALIZABLE_NAME = "serializable object";
    public final static String TAG = MainActivity.class.getName();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
	super.onCreate(savedInstanceState);

	setContentView(R.layout.activity_main);

    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
	super.onRestoreInstanceState(savedInstanceState);
	Log.d(TAG, "onRestoreInstanceState");

	// section 1: parcelable object
	CustomParcelableObject parcelableObject;
	parcelableObject = (CustomParcelableObject) savedInstanceState
		.getParcelable(PARCELABLE_KEY);
	Log.d(TAG, RESTORED_LOG + parcelableObject.getName());

	// section 2: serializable object
	CustomSerializableObject serializableObject;
	serializableObject = (CustomSerializableObject) savedInstanceState
		.getSerializable(SERIALIZABLE_KEY);
	Log.d(TAG, RESTORED_LOG + serializableObject.getName());

	// section 3: parcelable object array
	CustomParcelableObject[] parcelableObjectArray;
	parcelableObjectArray = (CustomParcelableObject[]) savedInstanceState
		.getParcelableArray(PARCELABLE_ARRAY_KEY);
	for (int i = 0; i < 3; i++) {
	    Log.d(TAG, RESTORED_LOG + parcelableObjectArray[i].getName());
	}

	// section 4: serializable object array
	CustomSerializableObject[] serializableObjectArray;
	serializableObjectArray = (CustomSerializableObject[]) savedInstanceState
		.getSerializable(SERIALIZABLE_ARRAY_KEY);
	for (int i = 0; i < 3; i++) {
	    Log.d(TAG, RESTORED_LOG + serializableObjectArray[i].getName());
	}

	// section 5: parcelable object 2d array
	CustomParcelableObjectWrapper parcelableObject2DArray;
	parcelableObject2DArray = (CustomParcelableObjectWrapper) savedInstanceState
		.getParcelable(PARCELABLE_2D_ARRAY_KEY);
	for (int i = 0; i < 3; i++) {
	    for (int j = 0; j < 3; j++) {
		Log.d(TAG,
			RESTORED_LOG
				+ parcelableObject2DArray
					.getCustomParcelableObject(i, j)
					.getName());
	    }
	}

	// section 6: serializable object 2d array
	CustomSerializableObject[][] serializableObject2DArray;
	serializableObject2DArray = (CustomSerializableObject[][]) savedInstanceState
		.getSerializable(SERIALIZABLE_2D_ARRAY_KEY);
	for (int i = 0; i < 3; i++) {
	    for (int j = 0; j < 3; j++) {
		Log.d(TAG,
			RESTORED_LOG
				+ serializableObject2DArray[i][j].getName());
	    }
	}

	// section 7: object in application
	MyApplication myApplication = (MyApplication) getApplication();
	CustomObject[][] customObjectArray = myApplication
		.getCustomObjectArray();
	Log.d(TAG, myApplication.getObjectDescription(RESTORED_LOG));

    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
	super.onSaveInstanceState(outState);
	Log.d(TAG, "onSaveInstanceState");

	// section 1: parcelable object
	CustomParcelableObject parcelableObject = new CustomParcelableObject(
		PARCELABLE_NAME);
	outState.putParcelable(PARCELABLE_KEY, parcelableObject);
	Log.d(TAG, SAVED_LOG + parcelableObject.getName());

	// section 2: serializable object
	CustomSerializableObject serializableObject = new CustomSerializableObject(
		SERIALIZABLE_NAME);
	outState.putSerializable(SERIALIZABLE_KEY, serializableObject);
	Log.d(TAG, SAVED_LOG + serializableObject.getName());

	// section 3: parcelable object array
	CustomParcelableObject[] parcelableObjectArray = new CustomParcelableObject[3];
	for (int i = 0; i < 3; i++) {
	    parcelableObjectArray[i] = new CustomParcelableObject(
		    PARCELABLE_NAME + " at " + i);
	}
	outState.putParcelableArray(PARCELABLE_ARRAY_KEY, parcelableObjectArray);
	for (int i = 0; i < 3; i++) {
	    Log.d(TAG, SAVED_LOG + parcelableObjectArray[i].getName());
	}

	// section 4: serializable object array
	CustomSerializableObject[] serializableObjectArray = new CustomSerializableObject[3];
	for (int i = 0; i < 3; i++) {
	    serializableObjectArray[i] = new CustomSerializableObject(
		    SERIALIZABLE_NAME + " at " + i);
	}
	outState.putSerializable(SERIALIZABLE_ARRAY_KEY,
		serializableObjectArray);
	for (int i = 0; i < 3; i++) {
	    Log.d(TAG, SAVED_LOG + serializableObjectArray[i].getName());
	}

	// section 5: parcelable object 2d array
	CustomParcelableObjectWrapper parcelableObject2DArray = new CustomParcelableObjectWrapper(
		PARCELABLE_NAME);
	outState.putParcelable(PARCELABLE_2D_ARRAY_KEY, parcelableObject2DArray);
	for (int i = 0; i < 3; i++) {
	    for (int j = 0; j < 3; j++) {
		Log.d(TAG,
			SAVED_LOG
				+ parcelableObject2DArray
					.getCustomParcelableObject(i, j)
					.getName());
	    }
	}

	// section 6: serializable object 2d array
	CustomSerializableObject[][] serializableObject2DArray = new CustomSerializableObject[3][3];
	for (int i = 0; i < 3; i++) {
	    for (int j = 0; j < 3; j++) {
		serializableObject2DArray[i][j] = new CustomSerializableObject(
			SERIALIZABLE_NAME + " at " + i + ", " + j);
	    }
	}
	outState.putSerializable(SERIALIZABLE_2D_ARRAY_KEY,
		serializableObject2DArray);
	for (int i = 0; i < 3; i++) {
	    for (int j = 0; j < 3; j++) {
		Log.d(TAG,
			SAVED_LOG + serializableObject2DArray[i][j].getName());
	    }
	}

	// section 7: object in application
	MyApplication myApplication = (MyApplication) getApplication();
	myApplication.setCustomObjectArray();
	Log.d(TAG, myApplication.getObjectDescription(SAVED_LOG));

    }

}
